@@ -7,9 +7,10 @@ this library, and provides conversion and importing methods from popular
 graph formats.
 """
 import uuid
-from itertools import product
+import itertools as it
 import numpy as np
 from scipy.spatial import cKDTree
+from graphdot.codegen.typetool import common_min_type
 from graphdot.graph.adjacency.atomic import AtomicAdjacency
 from graphdot.minipandas import DataFrame
 
@@ -50,6 +51,100 @@ class Graph:
                    repr(self.edges),
                    repr(self.title))
 
+    @staticmethod
+    def has_unified_types(graphs):
+        '''Check if all graphs have the same set of nodal/edge attributes.'''
+        first = next(iter(graphs))
+        node_t = first.nodes.rowtype()
+        edge_t = first.edges.rowtype()
+        for second in graphs:
+            if second.nodes.rowtype() != node_t:
+                return ('nodes', first, second)
+            elif second.edges.rowtype() != edge_t:
+                return ('edges', first, second)
+        return True
+
+    @classmethod
+    def unify_datatype(cls, graphs, inplace=False):
+        '''Ensure that each attribute has the same data type across graphs.
+
+        Parameters
+        ----------
+        graphs: list
+            A list of graphs that have the same set of node and edge
+            attributes. The types for each attribute will then be
+            chosen to be the smallest scalar type that can safely hold all the
+            values as found across the graphs.
+        inplace: bool
+            Whether or not to modify the graph attributes in-place.
+
+        Returns
+        -------
+        None or list
+            If inplace is True, the graphs will be modified in-place and
+            nothing will be returned. Otherwise, a new list of graphs with
+            type-unified attributes will be returned.
+        '''
+        if inplace is not True:
+            def shallowcopy(g):
+                h = cls(
+                    nodes=g.nodes.copy(deep=False),
+                    edges=g.edges.copy(deep=False),
+                    title=g.title
+                )
+                for key, val in g.__dict__.items():
+                    if key not in ['nodes', 'edges', 'title']:
+                        h.__dict__[key] = val
+                return h
+            graphs = [shallowcopy(g) for g in graphs]
+
+        attributes = {}
+        for component in ['nodes', 'edges']:
+            first = None
+            for g in graphs:
+                second = set(getattr(g, component).columns)
+                first = first or second
+                if second != first:
+                    raise TypeError(
+                        f'Graph {g} with node attributes {second} '
+                        'does not match with the other graphs.'
+                    )
+            attributes[component] = first
+        for component in ['nodes', 'edges']:
+            group = [getattr(g, component) for g in graphs]
+            for key in attributes[component]:
+                types = [g[key].get_type(concrete=True) for g in group]
+                t = common_min_type.of_types(types)
+                if t == np.object:
+                    t = common_min_type.of_types(types, coerce=False)
+                if t is None:
+                    raise TypeError(
+                        f'Cannot unify attribute {key} containing mixed '
+                        'object types'
+                    )
+
+                # print(component, key, t)
+
+                if np.issctype(t):
+                    for g in group:
+                        g[key] = g[key].astype(t)
+                        # print(f'g[{key}].dtype', g[key].dtype)
+                elif t in [list, tuple, np.ndarray]:
+                    t_sub = common_min_type.of_values(
+                        it.chain.from_iterable(
+                            it.chain.from_iterable([g[key] for g in group])
+                        )
+                    )
+                    if t_sub is None:
+                        raise TypeError(
+                            f'Cannot find a common type for elements in {key}.'
+                        )
+                    for g in group:
+                        g[key] = [np.array(seq, dtype=t_sub) for seq in g[key]]
+
+        if inplace is not True:
+            return graphs
+
     # @classmethod
     # def from_auto(cls, graph):
     #     # import importlib
@@ -124,15 +219,9 @@ class Graph:
             if index == 0:
                 edge_attr = sorted(edge.keys())
             elif edge_attr != sorted(edge.keys()):
-                # raise TypeError(f'Edge {(i, j)} '
-                #                 f'attributes {edge.keys()} '
-                #                 f'inconsistent with {edge_attr}')
-                raise TypeError('Edge {} attributes {} '
-                                'inconsistent with {}'.format(
-                                    (i, j),
-                                    edge.keys(),
-                                    edge_attr
-                                ))
+                raise TypeError(f'Edge {(i, j)} '
+                                f'attributes {edge.keys()} '
+                                f'inconsistent with {edge_attr}')
 
         edge_df = DataFrame()
         edge_df['!i'], edge_df['!j'] = zip(*graph.edges.keys())
@@ -174,7 +263,7 @@ class Graph:
             nodes['charge'] = atoms.get_initial_charges().astype(np.float32)
 
         pbc = np.logical_and(atoms.pbc, use_pbc)
-        images = [(atoms.cell.T * image).sum(axis=1) for image in product(
+        images = [(atoms.cell.T * image).sum(axis=1) for image in it.product(
             *tuple([-1, 0, 1] if p else [0] for p in pbc))]
         x = atoms.get_positions()
         x_images = np.vstack([x + i for i in images])
